-----------------------------------------------------------------------------

local AloftManaData = Aloft:NewModule("ManaData", "AceEvent-2.0")

-----------------------------------------------------------------------------

AloftTagDirectAccessData.mana = true
AloftTagData.mana = { data = "mana", events = "Aloft:OnManaDataChanged" }
AloftTagDirectAccessData.maxMana = true
AloftTagData.maxMana = { data = "maxMana", events = "Aloft:OnManaDataChanged" }
AloftTagMethods.manaDeficit = function(aloftData, value)
	if value and aloftData.mana then
		return aloftData.maxMana - aloftData.mana
	else
		return nil
	end
end
AloftTagData.manaDeficit = { data = "mana", "maxMana", events = "Aloft:OnManaDataChanged" }
AloftTagMethods.manaPercent = function(aloftData, value)
	if value and aloftData.mana then
		return floor(aloftData.mana*100 / aloftData.maxMana)
	else
		return nil
	end
end
AloftTagData.manaPercent = { data = "mana", "maxMana", events = "Aloft:OnManaDataChanged" }

AloftTagDirectAccessData.powerType = true
AloftTagData.powerType = { data = "powerType", events = "Aloft:OnManaDataChanged" }

local colorTable =
{
	[0] = "c0c0ff",	-- Mana
	[1] = "ff8080",	-- Rage
	[2] = "ffd100", -- Focus
	[3] = "ffff80",	-- Energy
	[4] = "80ffff", -- Happiness
}
AloftTagMethods.powerTypeColor = function(aloftData, value)
	return value and aloftData.powerType and ("|cff%s%s|r"):format(colorTable[aloftData.powerType], value or "") or nil
end

AloftTagData.powerTypeColor = { data = "powerType", events = "Aloft:OnManaDataChanged" }

-----------------------------------------------------------------------------

function AloftManaData:ProvidesData()
	return "mana", "maxMana", "powerType"
end

function AloftManaData:RequiresData()
	return "unitid"
end

function AloftManaData:EnableDataSource()
	self:RegisterEvent("UNIT_MANA", "OnUnitManaChanged")
	self:RegisterEvent("UNIT_RAGE", "OnUnitManaChanged")
	self:RegisterEvent("UNIT_FOCUS", "OnUnitManaChanged")
	self:RegisterEvent("UNIT_ENERGY", "OnUnitManaChanged")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "OnUnitManaChanged")
	self:RegisterEvent("Aloft:OnUnitidDataChanged", "OnUnitidDataChanged")
end

-----------------------------------------------------------------------------

function AloftManaData:OnUnitManaChanged(unitid)
	local aloftData = Aloft:GetNameplate(UnitName(unitid))
	if aloftData and aloftData.unitid then
		aloftData.mana = UnitMana(unitid)
		aloftData.maxMana = UnitManaMax(unitid)
		aloftData.powerType = UnitPowerType(unitid)
		self:TriggerEvent("Aloft:OnManaDataChanged", aloftData)
	end
end

-----------------------------------------------------------------------------

function AloftManaData:UpdateData(aloftData)
	aloftData.mana = nil
	aloftData.maxMana = nil
	aloftData.powerType = nil

	if aloftData.unitid then
		aloftData.mana = UnitMana(aloftData.unitid)
		aloftData.maxMana = UnitManaMax(aloftData.unitid)
		aloftData.powerType = UnitPowerType(aloftData.unitid)
	end
end

function AloftManaData:OnUnitidDataChanged(aloftData)
	self:UpdateData(aloftData)
	self:TriggerEvent("Aloft:OnManaDataChanged", aloftData)
end

-----------------------------------------------------------------------------
